@@ -1,6 +1,9 @@
-# $Id: Construct.pm 39366 2013-05-06 10:11:00Z wsl $
+# $Id: Construct.pm 40904 2014-01-28 13:13:16Z wsl $
 # $URL: https://svn.uvt.nl/its-id/trunk/sources/xml-construct/lib/XML/LibXML/Construct.pm $
 
+use strict;
+use warnings FATAL => 'all';
+
 package XML::LibXML::Construct;
 
 use Carp qw(confess);
@@ -9,6 +12,10 @@ sub construct {
 	confess("Don't call XML::LibXML::Construct::construct; use either XML::LibXML->construct or \$doc->construct");
 }
 
+sub deconstruct {
+	confess("Don't call XML::LibXML::Construct::deconstruct; use either \$node->deconstruct or \$doc->deconstruct");
+}
+
 package XML::LibXML;
 
 require XML::LibXML;
@@ -50,14 +57,65 @@ sub construct {
 				confess("Reference type '$type' not supported");
 			}
 		} else {
-			$child = "$child";
-			utf8::upgrade($child);
-			$node->appendText($child);
+			my $c = $child;
+			utf8::upgrade($c);
+			$node->appendText($c);
 		}
 	}
 	return $node;
 }
 
+sub deconstruct { shift->documentElement->deconstruct }
+
+package XML::LibXML::DocumentFragment;
+
+sub deconstruct { map { $_->deconstruct } shift->childNodes }
+
+package XML::LibXML::Node;
+
+sub deconstruct { return }
+
+package XML::LibXML::Comment;
+
+# For some reason, XML::LibXML::Comment's parent class is XML::LibXML::Text,
+# even though the manpage seems to suggest it's XML::LibXML::Node.
+
+sub deconstruct { return }
+
+package XML::LibXML::Namespace;
+
+# For some reason, XML::LibXML::Namespace does not have a parent class,
+# even though the manpage seems to suggest it's XML::LibXML::Node.
+
+sub deconstruct { return }
+
+package XML::LibXML::Text;
+
+sub deconstruct { shift->nodeValue }
+
+package XML::LibXML::Attr;
+
+sub deconstruct {
+	my $self = shift;
+	my $name = $self->nodeName;
+	return if $name =~ /^xml/;
+	$name =~ s/^.*://;
+	return $name, $self->nodeValue;
+}
+
+package XML::LibXML::Element;
+
+sub deconstruct {
+	my $self = shift;
+	my $name = $self->nodeName;
+#	$name =~ s/^.*://;
+	my @element = ($name);
+	my %attrs = map { $_->deconstruct } $self->attributes;
+	push @element, \%attrs if %attrs;
+	push @element, map { $_->deconstruct } $self->childNodes;
+	return \@element;
+}
+
 1;
 
 __END__
@@ -122,14 +180,28 @@ XML::LibXML::Document will be created, filled and returned.
 Great care is taken to utf8::upgrade() every string before passing it to
 XML::LibXML to guarantee predictable behavior for any text string.
 
+=head2 $node->deconstruct
+
+Converts a DOM node back to a tree of perl data elements. Any comments,
+processing instructions, doctype declarations and namespace nodes are
+discarded.
+
+You can invoke this method on a document or an element. (In fact you can
+invoke it on any node, but the result might not be what you expect.)
+
+Note that elements are returned as arrayrefs.
+
+Document fragments are returned as a list of results, so be sure to call
+it in a list context.
+
 =head1 BUGS
 
 Namespaces are respected but otherwise unsupported; set up your document
-with namespaces beforehand if necessary. All created nodes are in the
-default (empty) namespace.
+with namespace declarations beforehand if necessary. Any namespace prefixes
+are passed through to XML::LibXML unmodified.
 
-If the official XML::LibXML package ever starts shipping a construct method
-of its own, it will collide rather fatally.
+If the official XML::LibXML package ever starts shipping a construct or
+deconstruct method of its own, it will collide rather fatally.
 
 =head1 EXAMPLES
 
